#ifndef GAMEMODEL_H
#define GAMEMODEL_H

/**
 * \file GameModel.h
 * \brief Déclaration de la classe GameModel
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

#include "Tir.h"
#include "Ennemi.h"
#include "Boss.h"
#include "Joueur.h"
#include "Bonus.h"
#include "Niveau.h"
#include "Horloge.h"

#include <vector>
#include <string>
#include <sstream>

using namespace std;

/**
 * \class GameModel GameModel.h
 * \brief Classe représentant le Modèle du jeu
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */
 
class GameModel
{
	private :
	
		int my_score;
		int my_viesMax;
		int my_difficulty;
		
		Joueur *my_ship;
		vector<Tir*> my_tirs;
		vector<Ennemi*> my_ennemis;
		vector<Bonus*> my_bonuses;
		vector<MovableElement*> my_deletedItems;

		TypeEnnemi* my_ennemi_types[NB_ENNEMY_TYPES];
		TypeTir* my_tirs_types[NB_TIRS_TYPES];
		TypeBonus* my_bonuses_types[NB_BONUS_TYPES];
		TypeBoss* my_boss_type;

		Boss *b;


		Niveau niveau;
		map<float, TypeEnnemi*>::const_iterator itNiveau;
		bool my_nextLevel;

		Horloge my_clock;
		
	
	public :
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param viesMax : Nombre maximum de vies du Joueur
		 * \param initialLives : Nombre initial de vies du Joueur
		 * \param difficulty : Difficulté du jeu
		 */	
		GameModel(int viesMax, int initialLives, int difficulty);

		/**
		 * \brief Destructeur
		 */  
		~GameModel();

		/**
		 * \brief Passer à "l'étape" suivante
		 *
		 * Méthode permettant de faire progresser le jeu
		 */  
		void nextStep();


		/**
		 * \brief Ajouter un Ennemi
		 *
		 * \param type : Type de l'Ennemi à ajouter
		 */  
		void addEnnemy(TypeEnnemi* type);
		
		/**
		 * \brief Ajouter un Tir
		 *
		 * \param t : Tir à ajouter
		 */  
		void addTir(Tir* t);
		
		/**
		 * \brief Ajouter un Bonus
		 *
		 * \param x : Abscisse du Bonus à ajouter
		 * \param y : Ordonnée du Bonus à ajouter
		 */  
		void addBonus(int x, int y);


		/**
		 * \brief Supprimer un Ennemi
		 *
		 * \param ennemi : Adresse de l'Ennemi à supprimer
		 */  
		void removeEnnemy(Ennemi* ennemi);
		
		/**
		 * \brief Supprimer un Ennemi
		 *
		 * \param it : Itérateur vers l'Ennemi à supprimer
		 */  
		void removeEnnemy(std::vector<Ennemi*>::iterator it);


		/**
		 * \brief Sauvegarder le score
		 *
		 * \param name : Pseudo à afficher
		 */
		void saveScore(string name);


		/**
		 * \brief Passer au Niveau suivant
		 *
		 * Vide le vector des Tir, réinitialise la Clock et redéfinit le Boss
		 */
		void changeNiveau();
		
		/**
		 * \brief Tuer le Joueur
		 *
		 * Méthode permettant de gérer la mort du Joueur
		 */
		void gameOver();
		
		
		//Accesseurs
		
		/**
		 * \brief Accesseur 
		 *
		 * Méthode récupérant la position du Joueur
		 *
		 * \param x : (Sortie seulement) Abscisse du Joueur
		 * \param y : (Sortie seulement) Ordonnée du Joueur
		 */
		void getShipPos(int &x, int &y) const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_nextLevel
		 *
		 * \return : Si l'on passe ou non au niveau suivant (\b true si oui, \b sinon)
		 */
		bool getNextLevel() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de niveau.my_id
		 *
		 * \return : L'identifiant du Niveau en cours
		 */
		int getLevelID() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_score
		 *
		 * \return : Le score actuel
		 */
		int getScore() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_difficulty
		 *
		 * \return : La difficulté du jeu
		 */
		int getDifficulty() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de niveau.my_id
		 *
		 * \return : L'identifiant du Niveau en cours, sous forme de chaîne de caractères
		 */
		string getStringLevelID() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_score
		 *
		 * \return : Le score actuel, sous forme de chaîne de caractères
		 */
		string getStringScore() const;

		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_ennemi_types
		 *
		 * \return : Le tableau contenant les différents TypeEnnemi
		 */
		TypeEnnemi* const* getEnnemyTypes() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_tirs_types
		 *
		 * \return : Le tableau contenant les différents TypeTir
		 */
		TypeTir* const*  getTirTypes() const ;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_boss_types
		 *
		 * \return : Le tableau contenant les différents TypeBoss
		 */
		TypeBoss* getBossType() const;

		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_ship
		 *
		 * \return : Le Joueur
		 */
		Joueur*  getPlayerShip() const;

		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_ennemis
		 *
		 * \return : Le vecteur contenant les Ennemi présents
		 */
		vector<Ennemi*>* getVectorEnnemis();
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_tirs
		 *
		 * \return : Le vecteur contenant les Tir présents
		 */
		vector<Tir*>* getVectorTir();
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_bonuses
		 *
		 * \return : Le vecteur contenant les Bonus présents
		 */
		const vector<Bonus*> & getVectorBonuses();
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_tirs
		 *
		 * \return : Le vecteur contenant les objets à supprimer
		 */
		vector<MovableElement*>* getDeletedItems();

		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_clock
		 *
		 * \return : L'Horloge du jeu
		 */
		Horloge*  getClock();

		
		/**
		 * \brief Définir le score
		 * \param score : Nouveau score
		 */
		void setScore(int score);
		
		/**
		 * \brief Définir si l'on passe au niveau suivant
		 * \param nextLvl : \b true si l'on passe au niveau suivant, \b false sinon
		 */
		void setNextLevel(bool nextLvl);

};

#endif
