#ifndef GAME_VIEW_H
#define GAME_VIEW_H

/**
 * \file GameView.h
 * \brief Déclaration de la classe GameView
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */
 
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
#include <map>
#include <vector>
#include "GraphicElement.h"
#include "MovableElement.h"
#include "TypeEnnemi.h"
#include "constantes.h"
#include "Interface.h"

class GameModel;

/**
 * \class GameView GameView.h
 * \brief Classe représentant la Vue du jeu
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

class GameView {
	private :
		int my_w, my_h;

		sf::RenderWindow *my_window;
		sf::Image *my_background_image;
        sf::Image *my_background_parallax;
		sf::Image *my_ship_image;
		sf::Image *my_shoot_image;
		sf::Image *my_boom_image;
		sf::Image *my_revive_image;
		sf::Image *my_bonus_image;
		std::map<TypeEnnemi* const, sf::Image*> my_ennemies_images; //Map qui contient les images des ennemis en fonction de leur type
		std::map<TypeTir* const, sf::Image*> my_tirs_images;

		GraphicElement my_background_sprite, my_background_sprite2;
        GraphicElement my_background_parallax_sprite, my_background_parallax_sprite2;

		Interface my_hud;
		std::string my_strings[NB_STRINGS];
		int my_langue;

		 //map qui contient tout les sprites;
		std::map<MovableElement * const, GraphicElement *> my_elementToGraphicElement;

		//sons
		sf::SoundBuffer *my_titlethemeBuffer;
		sf::SoundBuffer *my_deathmonsterBuffer;
		sf::SoundBuffer *my_deathbossBuffer;
		sf::SoundBuffer *my_nextLevelBuffer;
		sf::SoundBuffer *my_gameoverBuffer;
		sf::SoundBuffer *my_shotBuffer;

		sf::Music my_maintheme;
		sf::Sound my_titletheme;
		sf::Sound my_deathmonster;
		sf::Sound my_deathboss;
		sf::Sound my_nextLevel;
		sf::Sound my_gameover;
		sf::Sound my_shotsound;

		int my_soundLevel;
		int my_soundpack;

		GameModel *my_model;
		
		sf::Clock my_clock; // Clock qui sert pour le tir

	public :
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param w : Largeur de la Vue
		 * \param h : Hauteur de la Vue
		 * \param bpp : Bits par pixel (profondeur des couleurs)
		 */	
		GameView(int w, int h, int bpp);

		/**
		 * \brief Destructeur
		 */  
		~GameView();


		/**
		 * \brief Définir le Modèle
		 * \param model : GameModel à utiliser
		 */	  
		void setModel(GameModel * model);
		
		
		/**
		 * \brief Mettre à jour la langue
		 */  
		void updateLanguage();
		
		/**
		 * \brief Mettre à jour le volume sonore
		 */  
		void updateVolume();
		
		/**
		 * \brief Mettre à jour les sons utilisés
		 */  
		void updateSound();

		
		/**
		 * \brief Faire défiler l'arrière-plan
		 */  
		void bgscroll();
		
		/**
		 * \brief Dessiner une explosion
		 * \param elem : Sprite de l'Ennemi explosant
		 * \param adr : Ennemi explosant
		 */  
		void explosion(GraphicElement &elem, Ennemi* adr);

		
		/**
		 * \brief Dessiner tous les éléments dans la Vue
		 */  
		void draw();
		
		/**
		 * \brief Afficher les éléments dans une fenêtre
		 */  
		void Display();
		
		/**
		 * \brief Dessiner une transition
		 *
		 * Méthode permettant d'afficher un texte et un compte à rebours après la mort du Joueur ou avant le début d'un Niveau par exemple
		 *
		 * \param text : Texte de la transition
		 * \param t : Durée de la transition (en secondes)
		 */  
		void transition(std::string text, float t);
		
		/**
		 * \brief Synchroniser la Vue avec le Modèle
		 * \bug Une technique plus optimisée a été trouvée pour cette méthode, mais elle entraîne parfois des erreurs de segmentation, pour une raison inconnue
		 */  
		void synchronize();
		
		/**
		 * \brief Traite les événements de la SFML
		 * \param quit : (Sortie seulement) État du programme
		 */  
		bool treatEvents(bool &quit);

		
		/**
		 * \brief Dessiner l'écran de pause
		 */  
		int pause();

		
		/**
		 * \brief Dessiner l'introduction du jeu
		 */  
		void intro();
		
		/**
		 * \brief Dessiner le menu principal
		 */  
		int menu();
		
		/**
		 * \brief Dessiner le menu des options
		 * \param quit : (Sortie seulement) État du programme
		 * \param difficulty : (Sortie seulement) Difficulté du jeu
		 * \param initialLives : (Sortie seulement) Nombre initial de vies du Joueur
		 */  
		void options(bool &quit, int &difficulty, int &initialLives);
		
		/**
		 * \brief Dessiner le menu des meilleurs scores
		 * \param quit : (Sortie seulement) État du programme
		 * \bug Pour une raison inconnue, un Event::Closed se produit à l'ouverture du menu. Pour pallier à ça, il a fallu empêcher de fermer le programme "normalement" une fois sur cet écran (l'utilisation de la touche Escape est toujours possible cependant)
		 */  
		void viewHighscores(bool &quit);
		
		/**
		 * \brief Dessiner la demande de son pseudo à l'utilisateur
		 */  
		void askName();


};


#endif
