#ifndef JOUEUR_H
#define JOUEUR_H

/**
 * \file Joueur.h
 * \brief Déclaration de la classe Joueur
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

#include "Vaisseau.h"
#include "Ennemi.h"
#include <string>

class Tir;
class GameModel;

/**
 * \class Joueur Joueur.h
 * \brief Classe représentant le vaisseau contrôlé
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

class Joueur : public Vaisseau
{
	private :
		int my_vies;
		int my_power;
		int my_bombs;
		int my_numberOfShoots;
		
	public :
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param power : Puissance initiale du vaisseau contrôlé
		 * \param typeTir : Type de tir initial du vaisseau contrôlé
		 * \param vies : Nombre initial de vies du vaisseau contrôlé
		 * \param pvMax : Nombre maximum de points de vie du vaisseau contrôlé
		 * \param bombs : Nombre initial de bombes du vaisseau contrôlé
		 * \param shootingSpeed : Vitesse de tir initiale du vaisseau contrôlé
		 */
		Joueur(int power, TypeTir* typeTir, int vies, int pvMax, int bombs, float shootingSpeed); 


		/**
		 * \brief Déplacer le vaisseau contrôlé vers la gauche
		 */ 
		void moveLeft();
		
		/**
		 * \brief Déplacer le vaisseau contrôlé vers la droite
		 */ 
		void moveRight();
		
		/**
		 * \brief Déplacer le vaisseau contrôlé vers le bas
		 */ 
		void moveDown();
		
		/**
		 * \brief Déplacer le vaisseau contrôlé vers le haut
		 */ 
		void moveUp();

		
		/**
		 * \brief Faire tirer le vaisseau contrôlé
		 *
		 * \param model : Modéle du jeu
		 */
		void shoot(GameModel* model);
		
		/**
		 * \brief Indiquer le tir touchant le vaisseau
		 *
		 * \param model : Modéle du jeu
		 * \return : Le tir touchant le vaisseau contrôlé
		 */
		std::vector<Tir*>::iterator isShot(GameModel* model) const;
		
		/**
		 * \brief Indiquer si le vaisseau contrôlé est touché
		 *
		 * \param model : Modéle du jeu
		 * \return : L'état du vaisseau contrôlé (\b true si le vaisseau contrôlé est touché, \b false sinon)
		 */
		bool isHitByEnemy(GameModel* model);
		
		/**
		 * \brief Utiliser une bombe
		 *
		 * Méthode permettant de détruire tous les ennemis à l'écran
		 *
		 * \param model : Modéle du jeu
		 */
		void useBomb(GameModel* model);
		
		
		/**
		 * \brief Ajouter de la puissance
		 *
		 * \param power : Puissance à ajouter
		 * \param model : Modéle du jeu
		 */
		void addPower(int power, GameModel* model);
		
		/**
		 * \brief Ajouter des points de vie
		 *
		 * \param hp : Points de vie à ajouter
		 */
		void addHP(int hp);

		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_vies
		 *
		 * \return : Le nombre de vies actuel du vaisseau contrôlé
		 */
		int getVies() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_power
		 *
		 * \return : La puissance actuelle du vaisseau contrôlé
		 */
		int getPower() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_bombs
		 *
		 * \return : La quantité de bombes actuelle du vaisseau contrôlé
		 */
		int getBombs() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_numberOfShoots
		 *
		 * \return : Le nombre de tirs simultanés du vaisseau contrôlé (dépend de la puissance)
		 */
		int getNumberOfShoots() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_bombs
		 *
		 * \return : La quantité de bombes actuelle du vaisseau contrôlé, en chaîne de caractères
		 */
		std::string getStringBombs() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_power
		 *
		 * \return : La puissance actuelle du vaisseau contrôlé, en chaîne de caractères
		 */
		std::string getStringPower() const;

		 
		/**
		 * \brief Définir le nombre de vies du vaisseau contrôlé
		 */
		void setVies(int vies);

		/**
		 * \brief Définir la quantité de bombes du vaisseau contrôlé
		 */
		void setBombs(int bombs);

		/**
		 * \brief Définir la puissance du vaisseau contrôlé
		 */
		void setPower(int i);

		/**
		 * \brief Définir le nombre de tirs simultanés du vaisseau contrôlé
		 */
		void setNumberOfShoots(int n);
	
};

#endif
