#ifndef VAISSEAU_H
#define VAISSEAU_H

/**
 * \file Vaisseau.h
 * \brief Déclaration de la classe Vaisseau
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

#include <SFML/Window.hpp>
#include "MovableElement.h"
#include "Tir.h"
#include <vector>

class GameModel;
class TypeTir;

/**
 * \class Vaisseau Vaisseau.h
 * \brief Classe représentant les vaisseaux
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

class Vaisseau: public MovableElement
{
    private:
        int my_hp;
        int my_pvMax;
        bool my_isAlive;

        Vaisseau();

    protected:
        TypeTir* my_typeTir; /**< Pointeur vers le type des tirs du Vaisseau */
        sf::Clock my_clock; /**< Pendule interne au Vaisseau, pour y gérer le temps */
        float my_shootingSpeed; /**< Cadence de tir, en secondes (un tir toutes les x secondes) */

    public:
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param hp : Points de vie initiaux du Vaisseau
		 * \param type : Type de tir initial du Vaisseau
		 * \param shootingSpeed : Cadence de tir initiale du Vaisseau
		 */
        Vaisseau(int hp, TypeTir* type, float shootingSpeed);

		/**
		 * \brief Destructeur
		 */  
        virtual ~Vaisseau();


//Accesseurs
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_hp
		 *
		 * \return : Les points de vie actuels du Vaisseau
		 */
        int getHP() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_pvMax
		 *
		 * \return : Les points de vie maximum du Vaisseau
		 */
        int getMaxHP() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_typeTir
		 *
		 * \return : Le type de tir actuel du Vaisseau
		 */
        TypeTir* getTypeTir() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_isAlive
		 *
		 * \return : L'état actuel du Vaisseau (\b true si le Vaisseau n'est pas détruit, \b false sinon)
		 */
        bool getAlive() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_shootingSpeed
		 *
		 * \return : La cadence de tir actuelle du Vaisseau
		 */
        float getShootingSpeed() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_clock
		 *
		 * \return : La Clock interne au Vaisseau
		 */
        sf::Clock* getClock();

		
		/**
		 * \brief Définir les points de vie du Vaisseau
		 *
		 * \param i : Nouveaux points de vie du Vaisseau
		 */
        void setHP(int i);
		
		/**
		 * \brief Définir le type de tir du Vaisseau
		 *
		 * \param type : Nouveau type de tir du Vaisseau
		 */
        void setTypeTir(TypeTir* type);
		
		/**
		 * \brief Définir l'état du Vaisseau
		 *
		 * \param b : Nouvel état du Vaisseau (\b true si le Vaisseau n'est pas détruit, \b false sinon)
		 */
        void setAlive(bool b);


//Méthodes
		
		/**
		 * \brief Faire tirer le Vaisseau (méthode virtuelle pure)
		 * \param model : Modèle du jeu
		 */
        virtual void shoot(GameModel* model) = 0;
		
		/**
		 * \brief Indiquer le tir touchant le Vaisseau
		 *
		 * \param model : Modéle du jeu (méthode virtuelle pure)
		 * \return : Le tir touchant le vaisseau contrôlé
		 */
        virtual std::vector<Tir*>::iterator isShot(GameModel* model) const = 0;
};

#endif 
